<?php

namespace InSegment\ApiCore\Middleware;

use Closure;

use InSegment\ApiCore\Interfaces\ApiKeyInterface;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

class VerifyApiKey
{
    /**
     * Handle an incoming request.
     *
     * @param \Illuminate\Http\Request $request
     * @param \Closure  $next
     * @param string $apiKeyParam
     * @param string $authTypes
     * @return mixed
     */
    public function handle($request, Closure $next, $apiKeyParam, ...$authTypes)
    {
        if (in_array('route', $authTypes)) {
            $publicKey = $request->get($apiKeyParam);
        }
        
        if (empty($publicKey) && in_array('session', $authTypes)) {
            $publicKey = app('session')->get($apiKeyParam);
        }
        
        if (empty($publicKey)) {
            throw new UnauthorizedHttpException('');
        }
        
        $apiProvider = config('api_core.api_keys_class');
        if (!$apiProvider) {
            throw new UnauthorizedHttpException("Please, configure 'api_core.api_keys_class'");
        }
        
        $apiKey = $apiProvider::findByKey($publicKey);
        
        if (empty($apiKey) || !$apiKey instanceof ApiKeyInterface) {
            throw new UnauthorizedHttpException('Wrong public key');
        }
        
        $route = $request->route();
        if (is_array($route)) {
            $route = &$this->getArrayRouteRef(app());
            $route[2] += ['apiKey' => $apiKey];
        } else {
            $route->setParameter('apiKey', $apiKey);
        }
        
        return $next($request);
    }

    /**
     * Gets a reference to an array representing the route in App, instead of a copy
     * 
     * @param mixed $app
     * @return array
     */
    protected function &getArrayRouteRef($app)
    {
        return Closure::bind(function & () {
            return $this->currentRoute;
        }, $app, $app)->__invoke();
    }

}
